@@ -1,10 +1,14 @@
+from __future__ import unicode_literals
+
 from django.db import models
+from django.utils.encoding import python_2_unicode_compatible
 from django.utils.translation import ugettext_lazy as _
 
 from model_utils.models import TimeStampedModel, StatusModel, TimeFramedModel
-from model_utils.tracker import ModelTracker
+from model_utils.tracker import FieldTracker, ModelTracker
 from model_utils.managers import QueryManager, InheritanceManager, PassThroughManager
 from model_utils.fields import SplitField, MonitorField, StatusField
+from model_utils.tests.fields import MutableField
 from model_utils import Choices
 
 
@@ -14,6 +18,7 @@ class InheritanceManagerTestRelated(models.Model):
 
 
 
+@python_2_unicode_compatible
 class InheritanceManagerTestParent(models.Model):
     # FileField is just a handy descriptor-using field. Refs #6.
     non_related_field_using_descriptor = models.FileField(upload_to="test")
@@ -22,25 +27,46 @@ class InheritanceManagerTestParent(models.Model):
     normal_field = models.TextField()
     objects = InheritanceManager()
 
+    def __unicode__(self):
+        return unicode(self.pk)
+
+    def __str__(self):
+        return "%s(%s)" % (
+            self.__class__.__name__[len('InheritanceManagerTest'):],
+            self.pk,
+            )
+
 
 
 class InheritanceManagerTestChild1(InheritanceManagerTestParent):
     non_related_field_using_descriptor_2 = models.FileField(upload_to="test")
     normal_field_2 = models.TextField()
-    pass
+    objects = InheritanceManager()
+
 
 
 class InheritanceManagerTestGrandChild1(InheritanceManagerTestChild1):
     text_field = models.TextField()
 
 
+
+class InheritanceManagerTestGrandChild1_2(InheritanceManagerTestChild1):
+    text_field = models.TextField()
+
+
+
 class InheritanceManagerTestChild2(InheritanceManagerTestParent):
     non_related_field_using_descriptor_2 = models.FileField(upload_to="test")
     normal_field_2 = models.TextField()
-    pass
 
 
 
+class InheritanceManagerTestChild3(InheritanceManagerTestParent):
+    parent_ptr = models.OneToOneField(
+        InheritanceManagerTestParent, related_name='manual_onetoone',
+        parent_link=True)
+
+
 class TimeStamp(TimeStampedModel):
     pass
 
@@ -62,6 +88,18 @@ class Monitored(models.Model):
 
 
 
+class MonitorWhen(models.Model):
+    name = models.CharField(max_length=25)
+    name_changed = MonitorField(monitor="name", when=["Jose", "Maria"])
+
+
+
+class MonitorWhenEmpty(models.Model):
+    name = models.CharField(max_length=25)
+    name_changed = MonitorField(monitor="name", when=[])
+
+
+
 class Status(StatusModel):
     STATUS = Choices(
         ("active", _("active")),
@@ -153,12 +191,13 @@ class ByAuthorQuerySet(models.query.QuerySet, AuthorMixin):
 
 
 class FeaturedManager(models.Manager):
-    def get_query_set(self):
+    def get_queryset(self):
         kwargs = {}
         if hasattr(self, "_db"):
             kwargs["using"] = self._db
         return ByAuthorQuerySet(self.model, **kwargs).filter(feature=True)
 
+    get_query_set = get_queryset
 
 
 class DudeQuerySet(models.query.QuerySet):
@@ -177,9 +216,11 @@ class DudeQuerySet(models.query.QuerySet):
 
 
 class AbidingManager(PassThroughManager):
-    def get_query_set(self):
+    def get_queryset(self):
         return DudeQuerySet(self.model).abiding()
 
+    get_query_set = get_queryset
+
     def get_stats(self):
         return {
             "abiding_count": self.count(),
@@ -204,6 +245,13 @@ class Car(models.Model):
     objects = PassThroughManager(DudeQuerySet)
 
 
+class SpotManager(PassThroughManager):
+    def get_queryset(self):
+        return super(SpotManager, self).get_queryset().filter(secret=False)
+
+    get_query_set = get_queryset
+
+
 class SpotQuerySet(models.query.QuerySet):
     def closed(self):
         return self.filter(closed=True)
@@ -216,32 +264,90 @@ class Spot(models.Model):
     name = models.CharField(max_length=20)
     secure = models.BooleanField(default=True)
     closed = models.BooleanField(default=False)
+    secret = models.BooleanField(default=False)
     owner = models.ForeignKey(Dude, related_name='spots_owned')
 
-    objects = PassThroughManager.for_queryset_class(SpotQuerySet)()
+    objects = SpotManager.for_queryset_class(SpotQuerySet)()
 
 
 class Tracked(models.Model):
     name = models.CharField(max_length=20)
     number = models.IntegerField()
+    mutable = MutableField()
 
-    tracker = ModelTracker()
+    tracker = FieldTracker()
+
+
+class TrackedFK(models.Model):
+    fk = models.ForeignKey('Tracked')
+
+    tracker = FieldTracker()
+    custom_tracker = FieldTracker(fields=['fk_id'])
+    custom_tracker_without_id = FieldTracker(fields=['fk'])
 
 
 class TrackedNotDefault(models.Model):
     name = models.CharField(max_length=20)
     number = models.IntegerField()
 
-    name_tracker = ModelTracker(fields=['name'])
+    name_tracker = FieldTracker(fields=['name'])
+
+
+class TrackedNonFieldAttr(models.Model):
+    number = models.FloatField()
+
+    @property
+    def rounded(self):
+        return round(self.number) if self.number is not None else None
+
+    tracker = FieldTracker(fields=['rounded'])
 
 
 class TrackedMultiple(models.Model):
     name = models.CharField(max_length=20)
     number = models.IntegerField()
 
+    name_tracker = FieldTracker(fields=['name'])
+    number_tracker = FieldTracker(fields=['number'])
+
+
+class InheritedTracked(Tracked):
+    name2 = models.CharField(max_length=20)
+
+
+class ModelTracked(models.Model):
+    name = models.CharField(max_length=20)
+    number = models.IntegerField()
+    mutable = MutableField()
+
+    tracker = ModelTracker()
+
+
+class ModelTrackedFK(models.Model):
+    fk = models.ForeignKey('ModelTracked')
+
+    tracker = ModelTracker()
+    custom_tracker = ModelTracker(fields=['fk_id'])
+    custom_tracker_without_id = ModelTracker(fields=['fk'])
+
+
+class ModelTrackedNotDefault(models.Model):
+    name = models.CharField(max_length=20)
+    number = models.IntegerField()
+
+    name_tracker = ModelTracker(fields=['name'])
+
+
+class ModelTrackedMultiple(models.Model):
+    name = models.CharField(max_length=20)
+    number = models.IntegerField()
+
     name_tracker = ModelTracker(fields=['name'])
     number_tracker = ModelTracker(fields=['number'])
 
+class InheritedModelTracked(ModelTracked):
+    name2 = models.CharField(max_length=20)
+
 
 class StatusFieldDefaultFilled(models.Model):
     STATUS = Choices((0, "no", "No"), (1, "yes", "Yes"))
@@ -251,3 +357,8 @@ class StatusFieldDefaultFilled(models.Model):
 class StatusFieldDefaultNotFilled(models.Model):
     STATUS = Choices((0, "no", "No"), (1, "yes", "Yes"))
     status = StatusField()
+
+
+class StatusFieldChoicesName(models.Model):
+    NAMED_STATUS = Choices((0, "no", "No"), (1, "yes", "Yes"))
+    status = StatusField(choices_name='NAMED_STATUS')
